var Funcs = function (readyInt) {
    let ax = {};
    this.optCache = _[readyInt.id].optCache;
    this._check = function () {
        let v = _[readyInt.id].ox;
        ax = v;
        return (typeof v == "undefined" || (v !== 0 && !v)) !== true;
    };
    this._msg = function (funcName) {
        console.log("如需返回值，调用" + funcName + "接口时须增加一秒延时")
    }
    this.saveOpt = function (funcName, paramList) {
        if (this.optCache.has(funcName)) {
            this.optCache.get(funcName).push(paramList)
        } else {
            this.optCache.set(funcName, [paramList])
        }
    }

    // ---------------------------文档操作------------------------------------------------
    /**
     * 打开指定的本地或远程OFD或PDF文件。
     * @param filename{string} 可以是本地文件路径、远程文件路径（http链接）。
     * @param readOnly{boolean} 是否以只读模式打开。true-只读，false-非只读。
     * @param strSaveUrl{string} 远程保存地址。以只读模式打开时无效。不需要时传空。
     * @returns {boolean} 调用接口是否成功（true：成功，false：失败）。
     */
    this.openFile = function (filename, readOnly, strSaveUrl) {
        let paramNum = arguments.length;
        let funcName = "openFile";
        if (this._check()) {
            let result;
            switch (paramNum) {
                case 1:
                    result = ax.openFile(filename);
                    return result;
                case 2:
                    result = ax.openFile(filename, readOnly);
                    return result;
                case 3:
                    result = ax.openFile(filename, readOnly, strSaveUrl);
                    return result;
                default:
                    console.log("请检查openFile参数个数")
            }
        } else {
            this.saveOpt(funcName, [filename, readOnly, strSaveUrl])
            this._msg(funcName);
        }
    };

    /**
     * 打开指定的本地或远程OFD或PDF文件。
     * @param filename{string} 可以是本地文件路径、远程文件路径（http链接）。
     * @param param{boolean} 是否以只读模式打开。true-只读，false-非只读。
     * @returns {boolean} 调用接口是否成功（true：成功，false：失败）。
     * 兼容接口
     */
    this.openFile2 = function (filename, param) {
        this.openFile(filename, param);
    }

    /**
     * 打开指定的远程OFD或PDF文件。
     * @param open_url{string} 远程文件路径（http链接）。
     * @param save_url{string} 远程保存地址。以只读模式打开时无效。不需要时传空。
     * @param readeOnly{boolean} 是否以只读模式打开。true-只读，false-非只读。
     * @returns {boolean} 调用接口是否成功（true：成功，false：失败）。
     * 兼容接口
     */
    this.openurl = function (open_url, save_url, readeOnly) {
        this.openFile(open_url, readeOnly, save_url);
    }

    /**
     * 弹出打开文件对话框
     * @returns {*}
     */
    this.openFileDlg = function () {
        let funcName = "openFileDlg";
        if (this._check()) {
            return ax.openFile();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 打开base64文件流的文件
     * @param stream{string}  base64格式文件流
     * @returns {boolean} 打开成功返回true，打开失败返false
     */
    this.openBase64File = function (stream) {
        let funcName = "openBase64File";
        if (this._check()) {
            return ax.openBase64File(stream);
        } else {
            this.saveOpt(funcName, [stream])
            this._msg(funcName);
        }
    }

    /**
     * 打开接续文档。即将多个文档按顺序合成一个文档打开。
     * @param strFileInfo{string}  接续文档信息Json格式字符串。
     * @returns {int} 调用接口是否成功（0：成功，其他值：失败）。
     */
    this.openContinuousFile = function (strFileInfo) {
        let funcName = "openContinuousFile";
        if (this._check()) {
            return ax.openContinuousFile(strFileInfo);
        } else {
            this.saveOpt(funcName, [strFileInfo])
            this._msg(funcName);
        }
    }

    /**
     * 无参：保存当前打开的文档，编辑后保存为原始文件，其中包含编辑后变更的内容。
     * 单参：将当前打开的文件保存到指定位置，保存为远程文件时应使用HTTP通用文件上传协议。
     * @param path {string} 保存的位置，可以是本地文件，如“D：/files/xxx.ofd”；也可以是远程文件，如“http://localhost:88/files/xxxx.ofd”。
     * @returns {boolean}
     */
    this.saveFile = function (path) {
        let paramNum = arguments.length;
        let funcName = "saveFile";
        if (this._check()) {
            switch (paramNum) {
                case 0:
                    return ax.saveFile();
                case 1:
                    return ax.saveFile(path);
                default:
                    console.log("请检查saveFile参数个数")
            }
        } else {
            this.saveOpt(funcName, [path])
            this._msg(funcName);
        }
    };

    /**
     * 保存指定文档并返回保存结果、url请求返回信息、字节数、md5码等信息。
     * @param strURL{string} 要保存的文件地址，可为本地文件地址或远程文件地址。
     * @returns {string} Json字符串
     */
    this.saveFileRetResult = function (strURL) {
        let funcName = "saveFileRetResult";
        if (this._check()) {
            return ax.saveFileRetResult(strURL);
        } else {
            this.saveOpt(funcName, [strURL])
            this._msg(funcName);
        }
    }

    /**
     * 将当前打开的ofd文档通过对话框另存为本地文件。
     * @returns {*} 无
     */
    this.saveAs = function () {
        let funcName = "saveAs";
        if (this._check()) {
            return ax.saveAs();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 获取当前文件的base64文件流。
     * @returns {string} 当前打开文件的base64文件流（无任何前后缀）。
     */
    this.saveBase64File = function () {
        let funcName = "saveBase64File";
        if (this._check()) {
            return ax.saveBase64File();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 获取当前文件指定页面的base64文件流。
     * 特殊说明：仅支持OFD文件。页面范围是多个页面，返回的是这多个页面的base64文件流，不会以string数组的形式返回。
     * @param strPageIndex{string} 页面范围，从1开始，相邻页面中间用‘-’隔开，不相邻页面用‘,’隔开。
     * @returns {string} 当前文件指定页面的base64文件流（无任何前后缀）。
     */
    this.saveBase64FilebyIndex = function (strPageIndex) {
        let funcName = "saveBase64FilebyIndex";
        if (this._check()) {
            return ax.saveBase64FilebyIndex(strPageIndex);
        } else {
            this.saveOpt(funcName, [strPageIndex])
            this._msg(funcName);
        }
    }

    /**
     * 关闭当前文件
     * @param bAutoSave{boolean} 文件被修改后关闭时是否自动保存文件，true：自动保存文件并关闭；false：弹窗选择是否保存文件，缺省时值为false。
     * @returns {boolean} 调用接口是否成功（true：成功，false：失败）。
     */
    this.closeFile = function (bAutoSave = false) {
        let funcName = "closeFile";
        if (this._check()) {
            ax.closeFile(bAutoSave);
        } else {
            this.saveOpt(funcName, [bAutoSave])
            this._msg(funcName);
        }
    };

    /**
     * 关闭所有打开的文档。
     * @returns {*} 无
     */
    this.closeAllFiles = function () {
        let funcName = "closeAllFiles";
        if (this._check()) {
            return ax.closeAllFiles();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 获取当前打开的文档是否被修改的标记。
     * 特殊说明：仅支持OFD文件
     * @returns {*} 当前文档是否被修改（true：已修改，false：未修改）。
     */
    this.isDocumentModified = function () {
        let funcName = "isDocumentModified";
        if (this._check()) {
            return ax.isDocumentModified();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 获取当前打开的文档是否被修改的标记。共2个接口可供调用。
     * 特殊说明：仅支持OFD文件
     * @returns {*} 当前文档是否被修改（true：已修改，false：未修改）。
     */
    this.IsDocomentHasChangedFlag = function () {
        let funcName = "IsDocomentHasChangedFlag";
        if (this._check()) {
            return ax.isDocumentModified();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 获取当前打开的文档是否被修改的标记。
     * 特殊说明：仅支持OFD文件
     * @returns {*} 当前文档是否被修改（true：已修改，false：未修改）。
     */
    this.IsDocumentHasChangedFlag = function () {
        return this.isDocumentModified();
    }

    /**
     * 打开当前打开文档属性（元数据）对话框。可进行文档信息设置、元数据设置、权限修改等。
     * 特殊说明：仅支持OFD文件打开文档属性对话框。
     * @returns {*} 无
     */
    this.docPropertiesDlg = function () {
        let funcName = "docPropertiesDlg";
        if (this._check()) {
            return ax.docPropertiesDlg();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 打开当前打开文档元数据对话框。可进行元数据设置、权限修改等。
     * 特殊说明：仅支持OFD文件打开元数据对话框。
     * @returns {*} 无
     */
    this.getMetaDataWindow = function () {
        let funcName = "getMetaDataWindow";
        if (this._check()) {
            return ax.getMetaDataWindow();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 为当前打开的文档设置元数据。
     * 特殊说明：仅支持OFD文件设置元数据。
     * @param key{string} 元数据的名称。可自定义id。
     * @param value{string} 元数据的值。
     */
    this.setMetaData = function (key, value) {
        let funcName = "setMetaData";
        if (this._check()) {
            ax.setMetaData(key, value);
        } else {
            this.saveOpt(funcName, [key, value])
            this._msg(funcName);
        }
    }

    /**
     * 获取当前打开的文档的指定元数据的值。
     * 特殊说明：仅支持OFD文件获取元数据。
     * @param key{string} 元数据的名称。
     * @returns {string} 元数据的值
     */
    this.getMetaData = function (key) {
        let funcName = "getMetaData";
        if (this._check()) {
            return ax.getMetaData(key);
        } else {
            this.saveOpt(funcName, [key])
            this._msg(funcName);
        }
    }

    /**
     * 获取当前打开文档指定名称的公文语义标引的内容。
     * 特殊说明：仅支持OFD文件获取语义容。
     * @param tagId{string} 语义标引的唯一定位符，应符合文档内标引文件的实际结构，例如“公文体/版头/密级”，见GB/T 33476.1-2016。
     * @returns {string} 成功返回公文语义标引的内容，失败为空。
     */
    this.getTaggedText = function (tagId) {
        let funcName = "getTaggedText";
        if (this._check()) {
            return ax.getTaggedText(tagId);
        } else {
            this.saveOpt(funcName, [tagId])
            this._msg(funcName);
        }
    }

    //PDF导出OFD
    //[in]url:导出后ofd文件的目标位置,可以是本地文件，如“D：/files/xxxxx.ofd”；也可以是远程文件，如“http://localhost:8080/exm/Test1”。
    //[in]dpi:图像分辨率，默认96，范围[96,1200]
    //[out]导出成功返回true，失败返回false
    /**
     * 将当前打开的PDF文档导出为OFD文档
     * @param url{string} 导出后OFD文档的目标位置，可以是本地目录，也可以是远程目录；此外还可以指定OFD文档的文件名，若不指定则与当前PDF文档同名。
     * @param dpi{int} 生成OFD文件的分辩率，默认96，其他常用如：144、300、600、	1200，范围[96, 1200]。
     * @returns {boolean} 调用接口是否成功（true：成功，false：失败）。
     */
    this.pdf2ofd = function (url, dpi) {
        let funcName = "pdf2ofd";
        if (this._check()) {
            dpi = Number(dpi);
            return ax.pdf2ofd(url, dpi);
        } else {
            this.saveOpt(funcName, [url, dpi])
            this._msg(funcName);
        }
    }

    /**
     * pdf文件转成ofd文件
     * 特殊说明：插件加载后即可调用，与当前是否有文档打开无关。
     * @param pszPdfUrl {string} pdf的url或本地全路径。
     * @param PszOfdUrl {string} ofd的url或本地全路径。
     * @param nBegin {int} 固定传0。
     * @param nEnd {int} 固定传0。
     * @param nResolution {int} 生成OFD文件的分辩率，如96、144、300等，范围[96, 1200]。
     * @param nResv {int} 固定传0。
     * @returns {int} 0：成功；其它值：失败。
     */
    this.convertPdfToOfd = function (pszPdfUrl, PszOfdUrl, nBegin, nEnd, nResolution, nResv) {
        let funcName = "convertPdfToOfd";
        if (this._check()) {
            return ax.convertPdfToOfd(pszPdfUrl, PszOfdUrl, nBegin, nEnd, nResolution, nResv);
        } else {
            this.saveOpt(funcName, [pszPdfUrl, PszOfdUrl, nBegin, nEnd, nResolution, nResv])
            this._msg(funcName);
        }
    }

    /**
     * 无参：清除签批的内容，按照笔画进行擦除
     * 单参：模拟点击命令触发对应功能。
     * 双参：根据可配置参数执行模拟点击命令触发对应功能。
     * @param a{string} 模拟点击命令id。
     * @param b{string} 可配置参数json字符串。
     * @returns {boolean} 调用接口是否成功（true：成功，false：失败）
     */
    this.performClick = function (a, b) {
        let funcName = "performClick";
        let paramNum = arguments.length;
        if (this._check()) {
            switch (paramNum) {
                case 0:
                    return ax.performClick();
                case 1:
                    if(a == "f_saveas") return this.saveAs();
                    else if(a == "vzmode_original") return this.zoomReset();
                    else if(a == "eraser_middle") a = "t_tableteraser";
                    return ax.performClick(a);
                case 2:
                    if(a == "f_saveas") {
                        let obj = JSON.parse(b);
                        return this.saveFile(obj.filepath + '/' + obj.filename);
                    }
                    return ax.performClick(a, b);
            }
        } else {
            this.saveOpt(funcName, [a, b])
            this._msg(funcName);
        }
    }

    /**
     * 根据可配置参数执行模拟点击命令触发对应功能。
     * @param commandId{string} 模拟点击命令id。
     * @param paramJson{string} 可配置参数json字符串。
     * @returns {boolean} 调用接口是否成功（true：成功，false：失败）
     * 兼容接口
     */
    this.performClickConfigable = function (commandId, paramJson) {
        this.performClick(commandId, paramJson);
    }

    /**
     * 获取当前打开文档的总页数。
     * @returns {int} 当前打开文档的总页数。
     */
    this.getPageCount = function () {
        let funcName = "getPageCount";
        if (this._check()) {
            return ax.getPageCount();//2019.10.28 same
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 获取当前打开文档中关键字数量。
     * @param strKeyword{string} 需搜索的关键字字符串。
     * @param bbMatchWholeWord{bool}是否全词匹配。
     * @param bMatchCase{bool}是否区分大小写。
     * @returns {int}当前打开文档中关键字数量。未打开文档时调用返回-1。
     */
    this.getSearchResultAmount = function (strKeyword, bbMatchWholeWord, bMatchCase) {
        let funcName = "getSearchResultAmount";
        if (this._check()) {
            return ax.getSearchResultAmount(strKeyword, bbMatchWholeWord, bMatchCase);
        } else {
            this.saveOpt(funcName, [strKeyword, bbMatchWholeWord, bMatchCase])
            this._msg(funcName);
        }
    }

    /**
     * 获取当前打开文档中关键字的坐标信息
     * @param strKeyword{string} 需搜索的关键字字符串
     * @param pageIndex{int} 需搜索的页码
     * @param bMatchWholeWord{bool} 是否全词匹配。
     * @param bMatchCase{bool}是否区分大小写。
     * @returns {string}当前打开文档中关键字的坐标信息字符串
     */
    this.getTextCoordinates = function (strKeyword, pageIndex, bMatchWholeWord, bMatchCase) {
        let funcName = "getTextCoordinates";
        if (this._check()) {
            return ax.getTextCoordinates(strKeyword, pageIndex, bMatchWholeWord, bMatchCase);
        } else {
            this.saveOpt(funcName, [strKeyword, pageIndex, bMatchWholeWord, bMatchCase])
            this._msg(funcName);
        }
    }

    /**
     * 获取当前打开的OFD或PDF文档中的当前页文本内容。
     * @param pageIndex {int} 需获取文本的页面索引，从0开始。
     * @returns {string} 当前页面的文本内容字符串。
     */
    this.getPageTxt = function (pageIndex) {
        let funcName = "getPageTxt";
        if (this._check()) {
            return ax.getPageTxt(pageIndex);
        } else {
            this.saveOpt(funcName, [pageIndex]);
            this._msg(funcName);
        }
    }

    /**
     * 获取当前打开的OFD或PDF文档中的全部文本内容。共2个接口可供调用。
     * @returns {string} 文档中的全部文本内容字符串。
     */
    this.exportText = function () {
        let funcName = "exportText";
        if (this._check()) {
            return ax.getDocumentText();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 获取当前打开的OFD或PDF文档中的全部文本内容。共2个接口可供调用。
     * @returns {*} 文档中的全部文本内容字符串。
     */
    this.getDocumentText = function () {
        let funcName = "getDocumentText";
        if (this._check()) {
            return ax.getDocumentText();
        } else {
            this.saveOpt(funcName, []);
            this._msg(funcName);
        }
    }

    /**
     * 合并两个及以上OFD文件为一个新的OFD文件。
     * 特殊说明：仅支持OFD文件合并。
     * @param strJson {string} 合并文件信息的json字符串。
     * @returns {int} 0：成功；其它值：失败。
     */
    this.mergeFiles = function (strJson) {
        let funcName = "mergeFiles";
        if (this._check()) {
            return ax.mergeFiles(strJson);
        } else {
            this.saveOpt(funcName, [strJson])
            this._msg(funcName);
        }
    }

    /**
     * 将附件导入至出当前打开的OFD文档里。
     * 特殊说明：仅支持OFD文件导入附件。
     * @param strFilePath{string} 本地附件的绝对路径
     * @returns {int} 1：成功；0：失败。
     */
    this.ImportAttachment = function (strFilePath) {
        let funcName = "ImportAttachment";
        if (this._check()) {
            return ax.ImportAttachment(strFilePath);
        } else {
            this.saveOpt(funcName, [strFilePath])
            this._msg(funcName);
        }
    }

    // ---------------------------回调函数------------------------------------------------
    /**
     * 注册监听函数
     * @param key{string} 需要被设置回调的参数名。
     * @param func{string} 回调的js函数名称
     * @param isAfter{boolean} 是否监听事后事件，true 为事后监听，false为事前监听
     * @return {string} 关键字字符串
     */
    this.registListener = function(key, func, isAfter) {
        let funcName = "registListener";
        if (this._check()) {
            return ax.registListener(key, func, Boolean(JSON.parse(isAfter)));
        } else {
            this.saveOpt(funcName, [key, func, isAfter]);
            this._msg(funcName);
        }
    }
    /**
     * 回调函数，与上面registListener组合使用
     */
    window['callback'] = function (){
        console.log("回调成功")
    }
    /**
     * 注册监听函数
     * @param key{string} 需要被设置回调的参数名。
     * @param func{string} 回调的js函数名称
     * @param isAfter{boolean} 是否监听事后事件，true 为事后监听，false为事前监听
     * @return {string} 关键字字符串
     * 兼容接口
     */
    this.setCallback = function (key, func, isAfter){
        this.registListener(key, func, isAfter);
    }

    /**
     * 注销回调函数
     * @param funcId{string} 参数为registListener接口的返回值
     * @return {*} 无
     */
    this.unRegistListener = function(funcId) {
        let funcName = "unRegistListener";
        if (this._check()) {
            return ax.unRegistListener(funcId);
        } else {
            this.saveOpt(funcName, [funcId]);
            this._msg(funcName);
        }
    }
    // ---------------------------文档打印------------------------------------------------
    /**
     * 打开打印对话框，可预览打印效果，进行打印范围、打印处理、打印管理等设置。
     * @returns {boolean} 调用接口是否成功（true：成功，false：失败）。
     */
    this.openPrintDlg = function () {
        let funcName = "openPrintDlg";
        if (this._check()) {
            return ax.print();
        } else {
            this.saveOpt(funcName, []);
            this._msg(funcName);
        }
    }

    /**
     * 打开打印对话框，可预览打印效果，进行打印范围、打印处理、打印管理等设置。
     * @returns {boolean} 调用接口是否成功（true：成功，false：失败）。
     * 兼容接口
     */
    this.showPrintSetting = function () {
        this.openPrintDlg();
    }

    /**
     * 设置打印对话框中复选框组件的默认值（是否选中）。
     * 特殊说明：在下次启动插件时需要重新配置。打印内容设置仅OFD文档有效。
     * @param cmpName{string} 组件名称
     * @param bChecked{boolean} 是否选中，true表示选中，false表示不选中。
     * @returns {*} 无
     */
    this.setPrintCompositeChecked = function (cmpName, bChecked) {
        let funcName = "setPrintCompositeChecked";
        if (this._check()) {
            return ax.setPrintCompositeChecked(cmpName, bChecked);
        } else {
            this.saveOpt(funcName, [cmpName, bChecked])
            this._msg(funcName);
        }
    }

    /**
     * 设置打印对话框中复选框组件的默认值（是否选中）。
     * 特殊说明：在下次启动插件时需要重新配置。打印内容设置仅OFD文档有效。
     * @param cmpName{string} 组件名称
     * @param bChecked{boolean} 是否选中，true表示选中，false表示不选中。
     * @returns {*} 无
     * 兼容接口
     */
    this.setPrintCompsiteChecked = function (cmpName, bChecked) {
        this.setPrintCompositeChecked(cmpName, bChecked);
    }

    /**
     * 设置打印对话框中组件是否可用
     * 特殊说明：在下次启动插件时需要重新配置。打印内容设置仅OFD文档有效。
     * @param strCompNames{string} 组件名称
     * @param bEnable{boolean} 是否可用，true表示可用，false表示不可用。
     * @returns {*} 无
     */
    this.setPrintCompositeEnable = function (strCompNames, bEnable) {
        let funcName = "setPrintCompositeEnable";
        if (this._check()) {
            return ax.setPrintCompositeEnable(strCompNames, bEnable);
        } else {
            this.saveOpt(funcName, [strCompNames, bEnable]);
            this._msg(funcName);
        }
    }

    /**
     * 设置打印对话框中组件是否可用
     * 特殊说明：在下次启动插件时需要重新配置。打印内容设置仅OFD文档有效。
     * @param strCompNames{string} 组件名称
     * @param bEnable{boolean} 是否可用，true表示可用，false表示不可用。
     * @returns {*} 无
     * 兼容接口
     */
    this.setPrintCompsiteEnable = function (strCompNames, bEnable) {
        this.setPrintCompositeEnable(strCompNames, bEnable);
    }

    /**
     * 设置是否允许打印签章
     * 特殊说明：在下次启动插件时需要重新配置。仅OFD文档有效。
     * @param bEnable{boolean} 是否允许，true表示允许，false表示不允许。
     * @returns {*} 无
     * 兼容接口
     */
    this.setPrintSealEnable = function (bEnable) {
        this.setPrintCompositeChecked("print_seal", bEnable);
        this.setPrintCompositeEnable("print_seal", bEnable);
    }

    /**
     * 根据传入的参数控制打印对话框中的相关参数。
     * @param printParam{string} 打印参数Json字符串。
     */
    this.printSet = function (printParam) {
        let funcName = "printSet";
        if (this._check()) {
            let newParam = {}
            if (typeof printParam !== "object") {
                printParam = JSON.parse(printParam)
            }
            for (let key in printParam) {
                if (key === "nPrintCopies") {
                    newParam["PageCopies"] = printParam[key]
                } else {
                    newParam[key] = printParam[key]
                }
            }
            ax.setPrintSettings(JSON.stringify(newParam));
        } else {
            this.saveOpt(funcName, [printParam])
            this._msg(funcName);
        }
    }

    /**
     * 设置最大打印份数，设置了该值后，打印对话框中的打印份数将不超过此值。共2个接口可供调用。
     * @param nPrintCopies{int} 最大打印份数。
     * 兼容接口
     */
    this.setMaxCopies = function (nPrintCopies) {
        this.printSet("{\"PageCopiesMax\":" + String(nPrintCopies) + "}");
    }

    /**
     * 设置最大打印份数，设置了该值后，打印对话框中的打印份数将不超过此值。共2个接口可供调用。
     * @param nPrintCopies{int} 最大打印份数。
     * 兼容接口
     */
    this.setPrintCopies = function (nPrintCopies) {
        this.setMaxCopies(nPrintCopies);
    }

    /**
     * 设置打印机名，设置了该值后，打印对话框中的打印机将不可选。
     * @param strName{string} 最大打印份数。
     */
    this.setPrinterName = function (strName) {
        this.printSet("{\"PrinterName\":\"" + strName + "\"}");
        this.setPrintCompositeEnable("printer", false);
    }

    /**
     * 设置打印分辨率，设置了该值后，设置了该值后，打印对话框中的打印质量控制按钮（一般/标清/高清/超清）将不可选。
     * @param dpi{int} 打印分辨率。
     */
    this.setPrintResolution = function (dpi) {
        this.printSet("{\"PrintDpi\":" + String(dpi) + "}");
        this.setPrintCompositeEnable("resolution", false);
    }

    /**
     * 根据传入的参数控制打印对话框中的相关参数。
     * @param printSetting{string} 打印参数Json字符串。
     * 兼容接口
     */
    this.printFileBySetting = function (printSetting) {
        this.printSet(printSetting)
    }

    /**
     * 直接打印文件（不弹出打印对话框），并按指定的打印任务名称和灰度打印设置执行。
     * 该接口已兼容suwell，有两种使用方式
     * 1. 第一个参数为文件名，第二个参数为是否灰度（类型为boolean，true表示灰度打印，false表示正常打印）
     * 2. 第一个参数为打印份数，第二个参数为文件名
     * @param param1{*}
     * @param param2{*}
     * @return{boolean} 调用接口是否成功 (true: 成功，false: 失败)
     */
    this.printFile = function (param1, param2) {
        let funcName = "printFile";
        if (this._check()) {
            let info = {}
            if (!isNaN(parseFloat(param1)) && isFinite(param1)) {//兼容接口
                info["PageCopies"] = Number(param1);
                info["DocName"] = param2;
                info["QuietPrint"] = true;
            } else {//标准接口
                info["DocName"] = param1;
                info["GrayscalePrint"] = JSON.parse(param2);
                info["QuietPrint"] = true;
            }
            return ax.printBySettings(JSON.stringify(info));
        } else {
            this.saveOpt(funcName, [param1, param2])
            this._msg(funcName);
        }
    }

    /**
     * 以指定份数直接打印文件（不弹出打印对话框）。
     * @param nCopies{int} 打印份数。
     */
    this.printFileCopies = function (nCopies) {
        this.printFile(nCopies, "");
    }

    /**
     * 直接打印文件，并按指定的打印任务名称和灰度打印设置执行。
     * @param docName{string} 发送到打印机的打印任务名称。
     * @param bGrayPrint{boolean} 是否灰度打印，true表示灰度打印，false表示正常打印。
     * @return{boolean} 调用接口是否成功（true：成功，false：失败）。
     * 兼容接口
     */
    this.printFile3 = function (docName, bGrayPrint) {
        return this.printFile(docName, bGrayPrint);
    }

    // ---------------------------界面配置------------------------------------------------
    /**
     * 显示或隐藏当前文档的工具栏。
     * @param bHide{boolean} 控制显示或隐藏，true-显示，false-隐藏。
     * @returns {*} 无
     */
    this.onOffToolBar = function (bHide) {
        let funcName = "onOffToolBar";
        if (this._check()) {
            return ax.onOffToolBar(bHide);
        } else {
            this.saveOpt(funcName, [bHide])
            this._msg(funcName);
        }
    }

    /**
     * 显示或隐藏插件的工具栏，设置后对所有文档生效，在插件关闭前永久保持生效。
     * 特殊说明：调用该接口后，设置onOffToolBar、onOffFileTool、onOffZoomTool、onOffZoomModeTool、onOffPageTool、onOffSelectTool、onOffSearchTool、onOffSignAndWaterMrksTool、onOffAnnoTool 、setCompositeVisible('toolbar','')临时配置工具栏无效，通过onOffToolMenuPermanently配置工具栏中子工具栏生效。
     * @param bHide{boolean} 控制显示或隐藏，true-显示，false-隐藏。
     * @returns {*} 无
     */
    this.onOffToolBarPermanently = function (bHide) {
        let funcName = "onOffToolBarPermanently";
        if (this._check()) {
            return ax.onOffToolBarPermanently(bHide);
        } else {
            this.saveOpt(funcName, [bHide])
            this._msg(funcName);
        }
    }

    /**
     * 获取当前UI界面工具栏的可见性。
     * @returns {boolean} true：可见；false：不可见。
     */
    this.isToolBarShowed = function () {
        let funcName = "isToolBarShowed";
        if (this._check()) {
            return ax.isToolBarShowed();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 显示或隐藏UI界面导航栏窗口。
     * @param bHide{boolean}  控制显示或隐藏，true-显示，false-隐藏。
     * @returns {*} 无
     */
    this.onOffNavBar = function (bHide) {
        let funcName = "onOffNavBar";
        if (this._check()) {
            return ax.onOffNavBar(bHide);
        } else {
            this.saveOpt(funcName, [bHide])
            this._msg(funcName);
        }
    }

    /**
     * 获取UI界面导航栏管理窗口的可见性。
     * @returns {boolean} true：可见；false：不可见。
     */
    this.navBarShowed = function () {
        let funcName = "navBarShowed";
        if (this._check()) {
            return ax.navBarShowed();
        } else {
            this.saveOpt(funcName, []);
            this._msg(funcName);
        }
    }

    /**
     * 显示或隐藏插件的菜单栏，设置后对所有文档生效，在插件关闭前永久保持生效。
     * 特殊说明：调用该接口后，设置onOffMenuBar、setCompositeVisible('menubar','')、setCompositeVisible('menu','')临时配置工具栏无效。
     * @param bHide{boolean} 控制显示或隐藏，true-显示，false-隐藏。
     * @returns {*} 无
     */
    this.onOffMenuBarPermanently = function (bHide) {
        let funcName = "onOffMenuBarPermanently";
        if (this._check()) {
            return ax.onOffMenuBarPermanently(bHide);
        } else {
            this.saveOpt(funcName, [bHide])
            this._msg(funcName);
        }
    }

    /**
     * 设置当前打开文档显示的视图选项，参数组合见下表。
     * 特殊说明：当前只支持导航栏的选项设置，且PDF文件只支持缩略图、全文搜索、开始工作、打开文件导览。
     * @param key{string} 表示首选项名称。
     * @param value{string} 表示首选项值。
     */
    this.setViewPreference = function (key, value) {
        let funcName = "setViewPreference";
        if (this._check()) {
            ax.setViewPreference(key, value);
        } else {
            this.saveOpt(funcName, [key, value])
            this._msg(funcName);
        }
    };

    /**
     * 显示或隐藏阅读软件界面的指定按钮或组件。
     * 特殊说明：如果是可控制界面元素列表之外的标识，不发生任何作用。
     * @param name{string} 按钮或组件标识。
     * @param visible{boolean} 是否可见，true表示可见，false表示不可见。
     * @returns {*} 无
     */
    this.setCompositeVisible = function (name, visible) {
        let funcName = "setCompositeVisible";
        if (this._check()) {
            if(name === "f_open") name = "open";
            else if(name === "file_save") name = "save";
            else if(name === "f_saveas") name = "saveas";
            else if(name === "file_print") name = "print";
            else if(name === "sign" || name === "d_sealsign") name = "sealsign";
            else if(name === "verify") name = "verifysign";
            else if(name === "menu") name = "menubar";
            return ax.setCompositeVisible(name, visible);
        } else {
            this.saveOpt(funcName, [name, visible])
            this._msg(funcName);
        }
    };

    /**
     * 显示或隐藏阅读软件界面的指定按钮或组件。
     * 特殊说明：如果是可控制界面元素列表之外的标识，不发生任何作用。
     * @param name{string} 按钮或组件标识。
     * @param visible{boolean} 是否可见，true表示可见，false表示不可见。
     * @returns {*} 无
     * 兼容接口
     */
    this.setCompsiteVisible = function (name, visible) {
        this.setCompositeVisible(name, visible);
    };


    /**
     * 启用或禁用阅读软件界面的指定按钮或组件。
     * 特殊说明：如果是可控制界面元素列表之外的标识，不发生任何作用。
     * @param name{string} 按钮或组件标识。
     * @param visible{boolean} 是否可见，true表示可见，false表示不可见。
     */
    this.setCompositeEnable = function (name, visible) {
        let funcName = "setCompositeEnable";
        if (this._check()) {
            ax.setCompositeEnable(name, visible);
        } else {
            this.saveOpt(funcName, [name, visible])
            this._msg(funcName);
        }
    };

    /**
     * 启用或禁用阅读软件界面的指定按钮或组件。
     * 特殊说明：如果是可控制界面元素列表之外的标识，不发生任何作用。
     * @param name{string} 按钮或组件标识。
     * @param visible{boolean} 是否可见，true表示可见，false表示不可见。
     * 兼容接口
     */
    this.setCompsiteEnable = function (name, visible) {
        this.setCompositeEnable(name, visible);
    };

    /**
     * 显示或隐藏插件中工具栏中指定子工具栏，设置后对所有文档生效，在插件关闭前永久保持生效。
     * 特殊说明：调用该接口后，设置onOffFileTool、onOffZoomTool、onOffZoomModeTool、onOffPageTool、onOffSelectTool、onOffSearchTool、onOffSignAndWaterMrksTool、onOffAnnoTool 临时配置工具栏无效；调用该接口隐藏所有子工具栏后，工具栏会被自动隐藏，同理在工具栏隐藏状态下，可通过调用该接口显示单独的子工具栏；通过onOffToolBarPermanently('true')设置工具栏状态会覆盖所有子工具栏的状态为显示，同理onOffToolBarPermanently('false')也会覆盖所有子工具栏状态为隐藏。
     * @param strToolName{string} 子工具栏名称。
     * @param bHide{boolean} 控制显示或隐藏，true-显示，false-隐藏。
     * @returns {*} 无
     */
    this.onOffToolMenuPermanently = function (strToolName, bHide) {
        let funcName = "onOffToolMenuPermanently";
        if (this._check()) {
            return ax.onOffToolMenuPermanently(strToolName, bHide);
        } else {
            this.saveOpt(funcName, [strToolName, bHide])
            this._msg(funcName);
        }
    }

    /**
     * 显示或隐藏UI界面工具栏中的文件子工具栏。
     * @param bHide{boolean} 控制显示或隐藏，true-显示，false-隐藏。
     * @returns {*} 无
     */
    this.onOffFileTool = function (bHide) {
        let funcName = "onOffFileTool";
        if (this._check()) {
            return ax.onOffFileTool(bHide);
        } else {
            this.saveOpt(funcName, [bHide])
            this._msg(funcName);
        }
    }

    /**
     * 获取工具栏中文件子工具栏的可见性。
     * @returns {boolean} true：可见；false：不可见。
     */
    this.isFileToolShowed = function () {
        let funcName = "isFileToolShowed";
        if (this._check()) {
            return ax.isFileToolShowed();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 显示或隐藏UI界面工具栏中的签章和水印子工具栏。
     * 特殊说明：仅支持当前打开为OFD文件时控制其显示或隐藏，PDF文件永久隐藏。
     * @param bHide{boolean} 控制显示或隐藏，true-显示，false-隐藏。
     * @returns {*} 无
     */
    this.onOffSignAndWaterMrksTool = function (bHide) {
        let funcName = "onOffSignAndWaterMrksTool";
        if (this._check()) {
            return ax.onOffSignAndWaterMrksTool(bHide);
        } else {
            this.saveOpt(funcName, [bHide])
            this._msg(funcName);
        }
    }

    /**
     * 获取UI界面工具栏中的签章和水印子工具栏的可见性。
     * @returns {boolean} true：可见；false：不可见。
     */
    this.isSignAndWaterMrksToolShowed = function () {
        let funcName = "isSignAndWaterMrksToolShowed";
        if (this._check()) {
            return ax.isSignAndWaterMrksToolShowed();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 显示或隐藏UI界面工具栏中的注释子工具栏。
     * 特殊说明：仅支持当前打开为OFD文件时控制其显示或隐藏，PDF文件永久隐藏。
     * @param bHide{boolean} 控制显示或隐藏，true-显示，false-隐藏。
     * @returns {*}
     */
    this.onOffAnnoTool = function (bHide) {
        let funcName = "onOffAnnoTool";
        if (this._check()) {
            return ax.onOffAnnoTool(bHide);
        } else {
            this.saveOpt(funcName, [bHide])
            this._msg(funcName);
        }
    }

    /**
     * 获取UI界面工具栏中的注释子工具栏的可见性。
     * @returns {boolean} true：可见；false：不可见。
     */
    this.isAnnoToolShowed = function () {
        let funcName = "isAnnoToolShowed";
        if (this._check()) {
            return ax.isAnnoToolShowed();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 显示或隐藏UI界面工具栏中的缩放模式子工具栏。
     * @param bHide{boolean} 控制显示或隐藏，true-显示，false-隐藏。
     * @returns {*} 无
     */
    this.onOffZoomModeTool = function (bHide) {
        let funcName = "onOffZoomModeTool";
        if (this._check()) {
            return ax.onOffZoomModeTool(bHide);
        } else {
            this.saveOpt(funcName, [bHide])
            this._msg(funcName);
        }
    }

    /**
     * 获取UI界面工具栏中的缩放模式子工具栏的可见性。
     * @returns {boolean} true：可见；false：不可见。
     */
    this.isZoomModeShowed = function () {
        let funcName = "isZoomModeShowed";
        if (this._check()) {
            return ax.isZoomModeShowed();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 显示或隐藏UI界面工具栏中的选择子工具栏。
     * @param bHide{boolean} 控制显示或隐藏，true-显示，false-隐藏。
     * @returns {*} 无
     */
    this.onOffSelectTool = function (bHide) {
        let funcName = "onOffSelectTool";
        if (this._check()) {
            return ax.onOffSelectTool(bHide);
        } else {
            this.saveOpt(funcName, [bHide])
            this._msg(funcName);
        }
    }

    /**
     * 获取UI界面工具栏中的选择子工具栏的可见性。。
     * @returns {boolean} true：可见；false：不可见。
     */
    this.isSelectToolShowed = function () {
        let funcName = "isSelectToolShowed";
        if (this._check()) {
            return ax.isSelectToolShowed();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 显示或隐藏UI界面工具栏中的搜索子工具栏。
     * @param bHide{boolean} 控制显示或隐藏，true-显示，false-隐藏。
     * @returns {*} 无
     */
    this.onOffSearchTool = function (bHide) {
        let funcName = "onOffSearchTool";
        if (this._check()) {
            return ax.onOffSearchTool(bHide);
        } else {
            this.saveOpt(funcName, [bHide])
            this._msg(funcName);
        }
    }

    /**
     * 获取UI界面工具栏中的搜索子工具栏的可见性。
     * @returns {boolean}  true：可见；false：不可见。
     */
    this.isSearchToolShowed = function () {
        let funcName = "isSearchToolShowed";
        if (this._check()) {
            return ax.isSearchToolShowed();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 显示或隐藏UI界面工具栏中的页码子工具栏。
     * @param bHide{boolean} 控制显示或隐藏，true-显示，false-隐藏。
     * @returns {*} 无
     */
    this.onOffPageTool = function (bHide) {
        let funcName = "onOffPageTool";
        if (this._check()) {
            return ax.onOffPageTool(bHide);
        } else {
            this.saveOpt(funcName, [bHide])
            this._msg(funcName);
        }
    }

    /**
     * 获取UI界面工具栏中的页码子工具栏的可见性。
     * @returns {boolean} true：可见；false：不可见。
     */
    this.isPageToolShowed = function () {
        let funcName = "isPageToolShowed";
        if (this._check()) {
            return ax.isPageToolShowed();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 显示或隐藏UI界面工具栏中的缩放子工具栏。
     * @param bHide{boolean} 控制显示或隐藏，true-显示，false-隐藏。
     * @returns {*} 无
     */
    this.onOffZoomTool = function (bHide) {
        let funcName = "onOffZoomTool";
        if (this._check()) {
            return ax.onOffZoomTool(bHide);
        } else {
            this.saveOpt(funcName, [bHide])
            this._msg(funcName);
        }
    }

    /**
     * 获取工具栏中缩放子工具栏的可见性。
     * @returns {boolean} true：可见；false：不可见。
     */
    this.isZoomToolShowed = function () {
        let funcName = "isZoomToolShowed";
        if (this._check()) {
            return ax.isZoomToolShowed();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 显示或隐藏UI界面导航栏中的子项。
     * @param strCompName{string} 导航栏中的子项名称，见下方导航栏子项标识列表。
     * @param strVisible{string} 控制显示或隐藏，"true"-显示，"false"-隐藏。
     * @returns {*} 无
     */
    this.setNavigatorVisible = function (strCompName, strVisible) {
        if(strCompName === "vn_customtag")
            strCompName = "vn_tags";
        else if(strCompName === "vn_signature")
            strCompName = "vn_eseal";
        else if(strCompName === "vn_fileversion")
            strCompName = "vn_versions";
        this.setCompositeVisible(strCompName, Boolean(JSON.parse(strVisible)));
    }

    /**
     * 设置文档显示模式。
     * @param bSingleMode{boolean} 控制是否为单文件模式，true-单文件，false-多文件。默认为多文件模式。
     * @returns {*} 无
     */
    this.setSingleMode = function (bSingleMode) {
        let funcName = "setSingleMode";
        if (this._check()) {
            return ax.setSingleMode(bSingleMode);
        } else {
            this.saveOpt(funcName, [bSingleMode])
            this._msg(funcName);
        }
    }

    // ---------------------------视图操作------------------------------------------------
    /**
     * 放大当前打开文档页面显示比例。每次调用后按10%、25%、50%、75%、100%、125%、150%、175%、200%、400%、800%逐级放大页面。
     * 特殊说明：多次调用后最大放大值为原始大小的8倍，插件中最大值可能小于此值。
     */
    this.zoomIn = function () {
        let funcName = "zoomIn";
        if (this._check()) {
            ax.zoomIn();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 缩小当前打开文档页面显示比例。每次调用后按800%、400%、200%、175%、150%、125%、100%、75%、50%、25%、10%逐级缩小页面。
     * 特殊说明：多次调用后最小缩小值为原始大小的10%，插件中最小值可能大于此值。
     */
    this.zoomOut = function () {
        let funcName = "zoomOut";
        if (this._check()) {
            ax.zoomOut();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 设置当前文档的页面显示比例
     * @param zoomValue{float} 页面显示比例的值，范围[10-800]，例如：120表示120%。
     */
    this.setZoomRadio = function (zoomValue) {
        let funcName = "setZoomRadio";
        if (this._check()) {
            ax.setZoomRadio(zoomValue);
        } else {
            this.saveOpt(funcName, [zoomValue])
            this._msg(funcName);
        }
    }

    /**
     * 设置当前文档的页面显示比例
     * @param zoomValue{float} 页面显示比例的值，范围[10-800]，例如：120表示120%。
     * 兼容接口
     */
    this.setScale = function (zoomValue) {
        this.setZoomRadio(zoomValue);
    }

    /**
     * 获取当前文档的页面显示比例。
     * @returns {float} 页面显示比例值，范围[10-800]，例如：120表示120%。
     */
    this.getZoomRadio = function () {
        let funcName = "getZoomRadio";
        if (this._check()) {
            return ax.getZoomRadio();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 设置当前打开文档的显示比例为原始大小，即100%比例。
     * @returns {*} 无
     */
    this.zoomReset = function () {
        let funcName = "zoomReset";
        if (this._check()) {
            return ax.zoomReset();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }
    /**
     * 设置当前打开文档的缩放模式。
     * @param zoomMode{int} 页面显示模式，0：原始（100%比例）；1：适页（以最大比例显示全部页面）；2：适宽（适合窗口宽度）；3：适高（适合窗口高度）。
     * @returns {*} 无
     */
    this.setZoomMode = function(zoomMode) {
        let funcName = "setZoomMode";
        if (this._check()) {
            let val = Number(zoomMode) ;
            if(val===1) this.fitPage();
            else if(val===2) this.fitWidth();
            else if(val===0) this.zoomReset();
            else if(val===3) this.fitHeight();
        } else {
            this.saveOpt(funcName, [zoomMode]);
            this._msg(funcName);
        }
    }


    /**
     * 设置当前打开文档的显示比例为适应窗口宽度。
     * @returns {*} 无
     */
    this.fitWidth = function () {
        let funcName = "fitWidth";
        if (this._check()) {
            return ax.fitWidth();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 设置当前打开文档的显示比例为适页，即完全以最大比例显示全部页面。
     * @returns {*} 无
     */
    this.fitPage = function () {
        let funcName = "fitPage";
        if (this._check()) {
            return ax.fitPage();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 设置当前打开文档的显示比例为适高，即完全以最大比例显示全部页面。
     * @returns {*} 无
     */
    this.fitHeight = function () {
        let funcName = "fitHeight";
        if (this._check()) {
            return ax.fitHeight();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 页码跳转到当前打开文档的首页
     * @returns {*} 无
     */
    this.gotoFirstPage = function () {
        let funcName = "gotoFirstPage";
        if (this._check()) {
            return ax.gotoFirstPage();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 页码跳转到当前打开文档的尾页。
     * @returns {*} 无
     */
    this.gotoLastPage = function () {
        let funcName = "gotoLastPage";
        if (this._check()) {
            return ax.gotoLastPage();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 页码跳转到当前打开文档的上一页。
     * @returns {*} 无
     */
    this.gotoPrevPage = function () {
        let funcName = "gotoPrevPage";
        if (this._check()) {
            return ax.gotoPrevPage();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 页码跳转到当前打开文档的下一页。
     * @returns {*} 无
     */
    this.gotoNextPage = function () {
        let funcName = "gotoNextPage";
        if (this._check()) {
            return ax.gotoNextPage();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 页码跳转到当前打开文档的指定页。共2个接口可供调用。
     * @param nPageNum{int} 表示要跳转的页码，从1开始。小于1则跳转到首页；大于总页数，则跳转到末页。
     * @returns {*} 无
     */
    this.gotoPage = function (nPageNum) {
        let funcName = "gotoPage";
        if (this._check()) {
            return ax.gotoPage(nPageNum);
        } else {
            this.saveOpt(funcName, [nPageNum])
            this._msg(funcName);
        }
    }

    /**
     * 页码跳转到当前打开文档的指定页。共2个接口可供调用。
     * @param index{int} 表示要跳转的页码，从1开始。小于1则跳转到首页；大于总页数，则跳转到末页。
     * @returns {*} 无
     */
    this.GotoPage = function (index) {
        let funcName = "GotoPage";
        if (this._check()) {
            return ax.gotoPage(parseInt(index));
        } else {
            this.saveOpt(funcName, [index])
            this._msg(funcName);
        }
    }

    /**
     * 页码跳转到当前打开文档的指定页。共2个接口可供调用。
     * @param nPage{int} 表示要跳转的页码，从1开始。小于1则跳转到首页；大于总页数，则跳转到末页。
     * @returns {*} 无
     */
    this.turnToPage = function (nPage) {
        let funcName = "turnToPage";
        if (this._check()) {
            return ax.gotoPage(nPage);
        } else {
            this.saveOpt(funcName, [nPage])
            this._msg(funcName);
        }
    }

    /**
     * 获取当前打开文档显示区域所在第一页的页码或者当前鼠标所在页的页码。页码从1开始。
     * @returns {int} 文档当前页码。从1开始。
     */
    this.getCurrentPageNum = function () {
        let funcName = "getCurrentPageNum";
        if (this._check()) {
            return ax.getCurrentPageNum();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 获取当前打开文档显示区域所在第一页或者当前鼠标所在页的页面索引。索引从0开始。
     * @returns {int}  文档当前页面索引。从0开始。
     */
    this.getCurrentPageIndex = function () {
        let funcName = "getCurrentPageIndex";
        if (this._check()) {
            return ax.getCurrentPageIndex();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 获取当前打开文档显示区域所在第一页的页码或者当前鼠标所在页的页码。页码从1开始。
     * @returns {int} 文档当前页码。从1开始。
     */
    this.GetCurPageIndex = function () {
        let funcName = "GetCurPageIndex";
        if (this._check()) {
            return ax.getCurrentPageNum();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 选择当前打开文档的全部文本，并高亮。
     * @returns {*} 无
     */
    this.selectAll = function () {
        let funcName = "selectAll";
        if (this._check()) {
            return ax.selectAll();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 设置全屏显示模式。
     * 特殊说明：需要退出全屏时将鼠标移至页面上中区域，会悬浮出关闭按钮，点击即可退出全屏模式。
     * @returns {*} 无
     */
    this.readerFullScreen = function () {
        let funcName = "readerFullScreen";
        if (this._check()) {
            return ax.readerFullScreen();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    // ---------------------------注释相关------------------------------------------------
    /**
     * 通过传入Json串直接插入文本批注。
     * @param strJson{string} 表示插入文本批注的内容信息的Json字符串。
     * @returns {string}  文本批注ID
     */
    this.addFreeText = function (strJson) {
        let funcName = "addFreeText";
        if (this._check()) {
            return ax.addFreeText(strJson);
        } else {
            this.saveOpt(funcName, [strJson])
            this._msg(funcName);
        }
    }

    /**
     * 通过ID获取文本批注内容。
     * @param id{string} 文本批注的id。
     * @returns {*} 文本批注内容信息的字符串。失败时返回空。
     */
    this.getFreeTextContentById = function (id) {
        let funcName = "getFreeTextContentById";
        if (this._check()) {
            return ax.getFreeTextContentById(id);
        } else {
            this.saveOpt(funcName, [id])
            this._msg(funcName);
        }
    }

    /**
     * 设置文本批注默认的字体信息。
     * 特殊说明：当返回值大于等于0时，json串设置的字体都会设置为默认值，设置出错的项使用系统默认项值。通过页面工具栏修改字体后此接口失效。
     * @param strJson{string}  表示文本批注所需的默认字体信息，默认字体通过json串的形式设置，未设置默认为字体或设置失败，默认字体为宋体。
     * @returns {int} 0：设置成功；其他：设置失败。
     */
    this.setFreeTextDefaultFont = function (strJson) {
        let funcName = "setFreeTextDefaultFont";
        if (this._check()) {
            return ax.setFreeTextDefaultFont(strJson);
        } else {
            this.saveOpt(funcName, [strJson])
            this._msg(funcName);
        }
    }

    /**
     * 控制鼠标在文本框上悬停时，文本框是否显示提示信息。
     * @param bSwitch{boolean} 控制是否显示，true-显示，false-不显示。
     * @returns {*} 无
     */
    this.AnnotaTextTitleSwitch = function (bSwitch) {
        let funcName = "AnnotaTextTitleSwitch";
        if (this._check()) {
            return ax.AnnotaTextTitleSwitch(bSwitch);
        } else {
            this.saveOpt(funcName, [bSwitch])
            this._msg(funcName);
        }
    }

    /**
     * 设置鼠标悬停在文本框上方时的提示信息
     * 特殊说明：需调用AnnotaTextTitleSwitch(true)功能后才会显示提示信息。
     * @param strJson {string} 文本提示信息Json字符串
     * @returns {*} 无
     */
    this.SetAnnotaTextTitle = function (strJson) {
        let funcName = "SetAnnotaTextTitle";
        if (this._check()) {
            return ax.SetAnnotaTextTitle(strJson);
        } else {
            this.saveOpt(funcName, [strJson])
            this._msg(funcName);
        }
    }

    /**
     * 设置文本批注时，字体下拉框内的可选字体。
     * 特殊说明:设置文本批注时，字体下拉框内的可选字体。
     * @param strFonts{string} 字体集合，多个字体之间用英文逗号间隔。
     * @returns {*} 无
     */
    this.SetFreeTextFontsGather = function (strFonts) {
        let funcName = "SetFreeTextFontsGather";
        if (this._check()) {
            return ax.SetFreeTextFontsGather(strFonts);
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 添加文本批注时字体下拉框内的可选字体。
     * 特殊说明：此设置会保存到临时配置文件，在下次启动时仍然生效。
     * @param strFonts{string} 字体集合，多个字体之间用英文逗号间隔。
     * @returns {*} 无
     */
    this.AddFreeTextFontsGather = function (strFonts) {
        let funcName = "AddFreeTextFontsGather";
        if (this._check()) {
            return ax.AddFreeTextFontsGather(strFonts);
        } else {
            this.saveOpt(funcName, [strFonts])
            this._msg(funcName);
        }
    }

    /**
     * 删除文本批注时字体下拉框内的可选字体。
     * 特殊说明: 此设置会保存到临时配置文件，在下次启动时仍然生效。
     * @param strFonts{string} 字体集合，多个字体之间用英文逗号间隔。
     * @returns {*} 无
     */
    this.SubFilterFontsGather = function (strFonts) {
        let funcName = "SubFilterFontsGather";
        if (this._check()) {
            return ax.SubFilterFontsGather(strFonts);
        } else {
            this.saveOpt(funcName, [strFonts])
            this._msg(funcName);
        }
    }

    /**
     * 设置签批笔迹粗细。
     * @param size{float} 笔迹尺寸，单位为mm。
     * @returns {*} {boolean} 调用接口是否成功（true：成功，false：失败）
     */
    this.setSignSize = function (size) {
        let funcName = "setSignSize";
        if (this._check()) {
            return ax.setSignSize(size);
        } else {
            this.saveOpt(funcName, [size])
            this._msg(funcName);
        }
    }

    /**
     * 设置时间戳格式。
     * @param startTime{string} 描述时间戳格式以及其字体信息的Json字符串
     * @returns {*} 无
     */
    this.setTimeFormat = function (startTime) {
        let funcName = "setTimeFormat";
        if (this._check()) {
            return ax.setTimeFormat(startTime);
        } else {
            this.saveOpt(funcName, [startTime])
            this._msg(funcName);
        }
    }

    /**
     * 设置时间戳格式。
     * @param data{string} 描述时间戳格式以及其字体信息的Json字符串
     * @returns {*} 无
     * 兼容接口
     */
    this.setDateFormat = function (data) {
        this.setTimeFormat(data)
    }

    /**
     * 通过文本框插入文本时，设置文本框上侧的字体设置栏中的常用语选择框。
     * @param strJson{string} 常用语。通过json字符串数组的形式插入，当json数组设置为空数组时，常用语选择栏置空不显示，只有设置了常用语才显示该项。
     * @returns {*} 无
     */
    this.SetFreqUserdComments = function (strJson) {
        let funcName = "SetFreqUserdComments";
        if (this._check()) {
            return ax.SetFreqUserdComments(strJson);
        } else {
            this.saveOpt(funcName, [strJson])
            this._msg(funcName);
        }
    }

    /**
     * 通过文本框插入文本时，设置文本框上侧的字体设置栏中的常用语选择框。
     * @param strJson{string} 常用语。通过json字符串数组的形式插入，当json数组设置为空数组时，常用语选择栏置空不显示，只有设置了常用语才显示该项。
     * @returns {*} 无
     * 兼容接口
     */
    this.setFreqUsedComments = function (strJson) {
        this.SetFreqUserdComments(strJson);
    }

    /**
     * 设置签名功能为图章时的图片，在设置登录时，根据登录人员不同级别设置，无图章签名的可以不设置。
     * 特殊说明:需设置用户id后才能调用此功能。
     * @param strJson{string} 签名图片json字符串。
     * @returns {int} 0：设置成功，此时以用户id命名的图章会添加到电脑临时目录的同级xReader/signatureImg下，如C:\Users/lenovo/AppData/Local/xReader/signatureImg/1.jpg。其他：设置失败。
     */
    this.SetSignatureImageByJson = function (strJson) {
        let funcName = "SetSignatureImageByJson";
        if (this._check()) {
            return ax.SetSignatureImageByJson(strJson);
        } else {
            this.saveOpt(funcName, [strJson])
            this._msg(funcName);
        }
    }

    /**
     * 设置图章对话框中默认显示的橡皮章或署名章。
     * @param strJson{string} 图章信息Json字符串数组，数组的每个对象表示一个图章。
     * @returns {*} 无
     */
    this.setCommonSigns = function (strJson) {
        let funcName = "setCommonSigns";
        if (this._check()) {
            return ax.setCommonSigns(strJson);
        } else {
            this.saveOpt(funcName, [strJson])
            this._msg(funcName);
        }
    }

    /**
     * 无参:
     *      打开添加图章对话框
     *      特殊说明:当前有打开文档，且打开文档为ofd时有效。
     * 单参:
     *      将fileurl路径的图片当做橡皮章添加到当前ofd文件中，位置待定，调用后此图片会跟随鼠标移动，鼠标移动到合适的位置点击即可完成橡皮章的添加。
     * @param a{string} 文件路径，目前仅支持本地路径。
     * @returns {boolean}  调用接口是否成功（true：成功，false：失败）
     */
    this.addImageStamp = function (a) {
        let funcName = "addImageStamp";
        let paramNum = arguments.length;
        if (this._check()) {
            switch (paramNum) {
                case 0:
                    return ax.addImageStamp();
                case 1:
                    return ax.addImageStamp(a);
            }
        } else {
            this.saveOpt(funcName, [a])
            this._msg(funcName);
        }
    }

    /**
     * 直接将图片添加到当前打开文档中，当前添加人可拖动重新调整存放位置。
     * @param annotInfo{string}  图章信息Json字符串。
     * @returns {string} 一个描述插入图章成功或失败状态的json串。
     */
    this.addStampAnnot = function (annotInfo) {
        let funcName = "addStampAnnot";
        if (this._check()) {
            return ax.addStampAnnot(annotInfo);
        } else {
            this.saveOpt(funcName, [annotInfo])
            this._msg(funcName);
        }
    }

    /**
     * 添加图章会跟随鼠标移动到插入图章的位置，点击后插入图章。
     * @param strJson{string} 图章信息json字符串。
     * @returns {*} 无
     */
    this.addMovingStamp = function (strJson) {
        let funcName = "addMovingStamp";
        if (this._check()) {
            return ax.addMovingStamp(strJson);
        } else {
            this.saveOpt(funcName, [strJson])
            this._msg(funcName);
        }
    }

    /**
     * 将远程地址图章添加到橡皮图章列表中。
     * @param strUrls{string} 远程橡皮图章加载地址。可添加多个，用";"隔开。
     * @returns {*} 无
     */
    this.addStampUrl = function (strUrls) {
        let funcName = "addStampUrl";
        if (this._check()) {
            return ax.addStampUrl(strUrls);
        } else {
            this.saveOpt(funcName, [strUrls])
            this._msg(funcName);
        }
    }

    /**
     * 打开添加文号章对话框。
     * 特殊说明：当前有打开文档，且打开文档为ofd时有效。
     * @returns {*} 无
     */
    this.AddChapter = function () {
        let funcName = "AddChapter";
        if (this._check()) {
            return ax.AddChapter();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 直接将文号章添加到当前打开的文档中。提供以模板ID方式添加（已有Json模板文件），和以导入xml模板方式添加（生成Json模板文件）两种方式。
     * @param strJson{string}  文号章信息Json字符串。
     * @returns {string} 文号章模板ID字符串，即Json模板无后缀文件名。以导入xml模板方式添加时会在安装目录下"xReader\Resource\Stamp\docNumStamp\"生成x.Json模板文件。
     */
    this.AddChapterByIDAndUrl = function (strJson) {
        let funcName = "AddChapterByIDAndUrl";
        if (this._check()) {
            return ax.AddChapterByIDAndUrl(strJson);
        } else {
            this.saveOpt(funcName, [strJson])
            this._msg(funcName);
        }
    }

    /**
     * 直接将文号章添加到当前打开的文档中。提供以模板ID方式添加（已有Json模板文件），和以导入xml模板方式添加（生成Json模板文件）两种方式。
     * @param strJson{string}  文号章信息Json字符串。
     * @returns {string} 文号章模板ID字符串，即Json模板无后缀文件名。以导入xml模板方式添加时会在安装目录下"xReader\Resource\Stamp\docNumStamp\"生成x.Json模板文件。
     * 兼容接口
     */
    this.AddChapterBy = function (strJson) {
        this.AddChapterByIDAndUrl(strJson)
    }

    /**
     * 添加矩形块，如白条用于遮盖。
     * @param strPos{string} 添加位置的Json字符串。
     * @param r{int} 颜色r通道。
     * @param g{int} 颜色g通道。
     * @param b{int} 颜色b通道。
     * @returns {*} 无
     */
    this.insertFillRect = function (strPos, r, g, b) {
        let funcName = "insertFillRect";
        if (this._check()) {
            return ax.insertFillRect(strPos, r, g, b);
        } else {
            this.saveOpt(funcName, [strPos, r, g, b])
            this._msg(funcName);
        }
    }

    /**
     * 根据用户ID、环节标识、注释类型、页码范围获取注释的信息，包括文档中的手写、文本框、图章，包含处理人及处理时间。
     * @param param{string} 用户ID、环节标识、注释类型、页码范围json字符串。
     * @returns {string} 一个返回含有批注信息的json数组
     */
    this.getAnnotInfoBy = function (param) {
        let funcName = "getAnnotInfoBy";
        if (this._check()) {
            return ax.getAnnotInfoBy(param);
        } else {
            this.saveOpt(funcName, [param])
            this._msg(funcName);
        }
    }

    /**
     * 删除指定范围得签批和图章，（默认清除全部版式文件手写内容）。
     * @param startTime{string} 删除内容Json字符串。
     * @returns {*} 无
     */
    this.deleteAnnotationByParameter = function (startTime) {
        let funcName = "deleteAnnotationByParameter";
        if (this._check()) {
            return ax.deleteAnnotationByParameter(startTime);
        } else {
            this.saveOpt(funcName, [startTime])
            this._msg(funcName);
        }
    }

    /**
     * 清空当前用户在当前节点的手写、图章、文本、折线、线段、高亮等注释。
     * @returns {*} 无
     */
    this.clearHandwrittenAnnotation = function () {
        let funcName = "clearHandwrittenAnnotation";
        if (this._check()) {
            return ax.clearHandwrittenAnnotation();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

   /* /!** 无效
     * 设置指定环节中，同一用户之前添加的批注是否禁止移动、编辑、删除等其中一个或多个操作，其可以在打开文档前调用。
     * @param strJson{string} 环节控制Json字符串。
     * @returns {*} 无
     *!/
    this.SetStepsLimits = function (strJson) {
        let funcName = "SetStepsLimits";
        if (this._check()) {
            return ax.SetStepsLimits(strJson);
        } else {
            this.saveOpt(funcName, [strJson])
            this._msg(funcName);
        }
    }*/

    /**
     * 设置当前用户对当前打开的文档的批注操作权限。
     * 特殊说明：在下次打开文件后需要重新设置权限。
     * @param strJson{string} 批注操作权限Json字符串。
     * @returns {*} 无
     */
    this.setUserAnnotPermission = function (strJson) {
        let funcName = "setUserAnnotPermission";
        if (this._check()) {
            return ax.setUserAnnotPermission(strJson);
        } else {
            this.saveOpt(funcName, [strJson])
            this._msg(funcName);
        }
    }

    // ---------------------------安全相关------------------------------------------------
    /**
     * 查看当前ofd文档中是否存在签章。
     * @returns {*} 无
     */
    this.hasSign = function () {
        let funcName = "hasSign";
        if (this._check()) {
            return ax.hasSign();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 获取当前ofd文档中签章个数。
     * @returns {int} 当前ofd文档中签章个数。如果调用时插件中未打开OFD文档，返回-1。
     */
    this.getSignaturesCount = function () {
        let funcName = "getSignaturesCount";
        if (this._check()) {
            return ax.getSignaturesCount();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 批量验证当前ofd文档中的签章，并以对话框显示验证结果。
     * @param bOnline{boolean} 是否在线验证。
     * @param strSignIds{string} 指定多个签章标识时，英文逗号分隔的签名的标识队列（数字与导航栏中签章列表对应，从1开始）。为空或“all”时验证所有签名。
     * @returns {*} 无
     */
    this.validSignature = function (bOnline, strSignIds) {
        let funcName = "validSignature";
        if (this._check()) {
            return ax.validSignature(bOnline, strSignIds);
        } else {
            this.saveOpt(funcName, [bOnline, strSignIds]);
            this._msg(funcName);
        }
    }

    /**
     * 将当前带签章的文件脱密并下载到本地。
     * @param bBlack{boolean} true 红字变黑字印章变黑（黑章黑头），false 颜色不变。
     * @param fileName{string} 文件名，另存对话框里自动署名。
     * @returns {*}
     */
    this.decryptAllSeals = function (bBlack, fileName) {
        let funcName = "decryptAllSeals";
        if (this._check()) {
            ax.decryptAllSeals(bBlack, fileName);
        } else {
            this.saveOpt(funcName, [bBlack, fileName]);
            this._msg(funcName);
        }
    }

    /**
     * 全部内容脱密为黑色。
     * @param fileName{string} 文件名，另存对话框里自动署名。
     * @returns {*}
     * 兼容接口
     */
    this.decryptAlltoBlack = function (fileName) {
        this.decryptAllSeals(false, fileName)
    }

    /**
     * 在当前打开的OFD文档上进行印章脱密。
     * @param decryptInfo{string} 脱密信息Json字符串。
     * @returns {string} 若脱密后保存base64数据成功则返回含base64数据的Json字符串如：{"base64stream":"xxxxxxxxxxx"}，否则返回空。
     */
    this.decryptAllSealsEx = function (decryptInfo) {
        let funcName = "decryptAllSealsEx";
        if (this._check()) {
            return ax.decryptAllSealsEx(decryptInfo)
        } else {
            this.saveOpt(funcName, [decryptInfo])
            this._msg(funcName);
        }
    }

    /**
     * 设置将要应用的印章标识。
     * 设置了此值后，将不能对当前文档应用与该标识不匹配的电子印章，此函数常用于配合用章授权。
     * @param uid{string} 印章的标识。
     * @returns {*} 无
     */
    this.setSealId = function (uid) {
        let funcName = "setSealId";
        if (this._check()) {
            return ax.setSealId(uid);
        } else {
            this.saveOpt(funcName, [uid])
            this._msg(funcName);
        }
    }

    /**
     * 向插件中添加图像/文本水印追踪信息，如水印、份号等，该信息将用于显示或打印文档。
     * @param xmlParam{string} xml数据 表示要添加水印的属性。
     * @param bSave{boolean} 是否触发文件修改标志。true：触发，保存文件后下次打开水印仍然存在；false：不触发，保存文件后，下次打开不会有此水印。缺省时，值为true。
     */
    this.addTrackInfo = function (xmlParam, bSave = true) {
        let funcName = "addTrackInfo";
        if (this._check()) {
            ax.addTrackInfo(xmlParam, bSave);
        } else {
            this.saveOpt(funcName, [xmlParam, bSave])
            this._msg(funcName);
        }
    };

    /**
     * 清除当前文档中的水印信息。
     * 特殊说明：仅支持OFD文件清除水印信息。
     * @returns {*} 无
     */
    this.clearTrackInfo = function () {
        let funcName = "clearTrackInfo";
        if (this._check()) {
            return ax.clearTrackInfo();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 设置密级标志信息。
     * 特殊说明：仅支持OFD文件设置密标信息。
     * @param strMBInfo
     * @returns {*} 无
     */
    this.setMBInfo = function (strMBInfo) {
        let funcName = "setMBInfo";
        if (this._check()) {
            return ax.setMBInfo(strMBInfo);
        } else {
            this.saveOpt(funcName, [strMBInfo])
            this._msg(funcName);
        }
    }

    /**
     * 打开打印对话框
     */
    this.openSealDlg = function () {
        let funcName = "openSealDlg";
        if (this._check()) {
            ax.sign();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    // ---------------------------信息配置------------------------------------------------
    /**
     * 配置插件信息。
     * 特殊说明：未特殊说明的配置项，配置完成后，下次打开也会使用此配置。
     * @param configName{string} 配置项。
     * @param configValue{string} 配置值。
     * @returns {*} 无
     */
    this.setConfigInfo = function (configName, configValue) {
        let funcName = "setConfigInfo";
        if (this._check()) {
            if(configName === "reader.singlemode" || configName === "singlemode")
                return ax.setSingleMode(Boolean(JSON.parse(configValue)));
            else
                return ax.setConfigInfo(configName, configValue);
        } else {
            this.saveOpt(funcName, [configName, configValue])
            this._msg(funcName);
        }
    }

    /**
     * 获取配置信息。
     * @param str{string} 配置项。可取值tablet.fullsign.userwidth（签字笔粗细）、tablet.fullsign.color（签字笔颜色）、tablet.fullsign.pentype（签字笔类型）。
     * @returns {string} 配置信息字符串。
     */
    this.GetConfigInfo = function (str) {
        let funcName = "GetConfigInfo";
        if (this._check()) {
            return ax.GetConfigInfo(str);
        } else {
            this.saveOpt(funcName, [str])
            this._msg(funcName);
        }
    }

    /**
     * 获取配置信息。
     * @param str{string} 配置项。可取值tablet.fullsign.userwidth（签字笔粗细）、tablet.fullsign.color（签字笔颜色）、tablet.fullsign.pentype（签字笔类型）。
     * @returns {string} 配置信息字符串。
     * 兼容接口
     */
    this.getConfigInfo = function (str) {
        return this.GetConfigInfo(str);
    }

    /**
     * 设置当前的用户名（用于注释和操作日志）。
     * 特殊说明：关闭插件后置空。
     * @param uname{string} string，用户名。
     * @returns {boolean} 调用接口是否成功（true：成功，false：失败）。
     */
    this.setUserName = function (uname) {
        let funcName = "setUserName";
        if (this._check()) {
            return ax.setUserName(uname);
        } else {
            this.saveOpt(funcName, [uname])
            this._msg(funcName);
        }
    }

    /**
     * 获取当前的用户名。
     * @returns {string} 用户名字符串。
     */
    this.getUserName = function () {
        let funcName = "getUserName";
        if (this._check()) {
            return ax.getUserName();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 设置用户id。
     * 特殊说明：关闭插件后置空。
     * @param strUserID{string} 用户id。
     * @returns {boolean} 调用接口是否成功（true：成功，false：失败）。
     */
    this.setUserId = function (strUserID) {
        let funcName = "setUserId";
        if (this._check()) {
            return ax.setUserId(strUserID);
        } else {
            this.saveOpt(funcName, [strUserID])
            this._msg(funcName);
        }
    }

    /**
     * 设置用户id。
     * 特殊说明：关闭插件后置空。
     * @param strUserID{string} 用户id。
     * @returns {boolean} 调用接口是否成功（true：成功，false：失败）。
     * 兼容接口
     */
    this.setUserID = function (strUserID) {
        this.setUserId(strUserID);
    }

    /**
     * 设置用户id。
     * 特殊说明：关闭插件后置空。
     * @param strUserID{string} 用户id。
     * @returns {boolean} 调用接口是否成功（true：成功，false：失败）。
     * 兼容接口
     */
    this.setuserid = function (strUserID) {
        this.setUserId(strUserID);
    }

    /**
     * 获取用户id。
     * @returns {string} 用户id字符串。
     */
    this.getUserId = function () {
        let funcName = "getUserId";
        if (this._check()) {
            return ax.getUserId();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 获取用户id。
     * @returns {string} 用户id。
     * 兼容接口
     */
    this.getUserID = function () {
        return this.getUserId();
    }

    /**
     * 获取用户id。
     * @returns {string} 用户id。
     * 兼容接口
     */
    this.getuserid = function () {
        return this.getUserId();
    }

    /**
     * 设置数据标签（用于注释）。
     * 特殊说明：该信息将同setUserId一起用于注释操作。如同时调用了setUserId和setDataTag，则需要UserId，DataTag都相同，此用户才对此注释有操作权限。关闭插件后置空。
     * @param dataTag{string} 数据标签。
     * @returns {*} 无
     */
    this.setDataTag = function (dataTag) {
        let funcName = "setDataTag";
        if (this._check()) {
            return ax.setDatatag(dataTag);
        } else {
            this.saveOpt(funcName, [dataTag])
            this._msg(funcName);
        }
    }

    /**
     * 设置数据标签（用于注释）。
     * 特殊说明：该信息将同setUserId一起用于注释操作。如同时调用了setUserId和setDataTag，则需要UserId，DataTag都相同，此用户才对此注释有操作权限。关闭插件后置空。
     * @param dataTag{string} 数据标签。
     * @returns {*} 无
     * 兼容接口
     */
    this.setDatatag = function (dataTag) {
        this.setDataTag(dataTag);
    }

    /**
     * 获取数据标签（用于注释）。
     * @returns {string} 数据标签。
     */
    this.getDataTag = function () {
        let funcName = "getDataTag";
        if (this._check()) {
            return ax.getDatatag();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 设置数据标签（用于注释）。
     * @returns {string} 数据标签。
     * 兼容接口
     */
    this.getDatatag = function () {
        return this.getDataTag();
    }

    /**
     * 为插件设置日志服务地址，设置了日志服务地址后，写入日志文件的消息将同时按照标准HTTP协议写入日志服务。
     * @param url{string} 日志服务地址。
     */
    this.setLogSvrURL = function (url) {
        let funcName = "setLogSvrURL";
        if (this._check()) {
            ax.setLogSvrURL(url);
        } else {
            this.saveOpt(funcName, [url])
            this._msg(funcName);
        }
    }

    /**
     * 获取版式阅读软件日志文件路径，软件通过日志文件，将内部操作（如打印、签名、验签等）进行记录和汇报。
     * @returns {string} 成功则返回日志文件的路径，如“C:/Temps/reader.log”或者“/opt/reader.log”；失败则返回null。
     */
    this.getLogFilePath = function () {
        let funcName = "getLogFilePath";
        if (this._check()) {
            return ax.getLogFilePath();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 设置网络连接会话标识
     * @param strCookie{string} 网络身份标识字符串。
     * 兼容接口
     */
    this.setCookie = function (strCookie) {
        this.setConfigInfo("userinfo.usertoken", strCookie);
    }

    /**
     * 设置网络连接会话标识
     * @param strCookie{string} 网络身份标识字符串。
     * 兼容接口
     */
    this.setUserToken = function (strCookie) {
        this.setConfigInfo("userinfo.usertoken", strCookie);
    }

    /**
     * 提供该插件版本信息。对比版本号差异，判断是否更新版本。
     * @returns {string} 插件版本字符串信息。例：3.0.2024.0101，2024.0101为版本更新日期。
     */
    this.getPluginVersion = function () {
        let funcName = "getPluginVersion";
        if (this._check()) {
            return ax.getPluginVersion();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 提供该插件版本信息。对比版本号差异，判断是否更新版本。
     * @returns {string} 插件版本字符串信息。例：3.0.2024.0101，2024.0101为版本更新日期。
     */
    this.version = function () {
        let funcName = "version";
        if (this._check()) {
            return ax.getPluginVersion();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 提供该插件版本信息。对比版本号差异，判断是否更新版本。
     * @returns {string} 插件版本字符串信息。例：3.0.2024.0101，2024.0101为版本更新日期。
     */
    this.getAppVersion = function () {
        let funcName = "getAppVersion";
        if (this._check()) {
            return ax.getPluginVersion();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 获取阅读器注册状态
     * @returns {int} 0：已注册，其它值表示未注册。
     */
    this.GetReaderRegStatus = function () {
        let funcName = "GetReaderRegStatus";
        if (this._check()) {
            return ax.GetReaderRegStatus();
        } else {
            this.saveOpt(funcName, []);
            this._msg(funcName);
        }
    }

    /**
     * 获取当前进程PID。
     * @returns {int} 当前进程PID。
     */
    this.GetCurrentAppPID = function () {
        let funcName = "GetCurrentAppPID";
        if (this._check()) {
            return ax.GetCurrentAppPID();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 获取本机IP和MAC地址信息。
     * @returns {string} IP和MAC地址json字符串。
     */
    this.getIPMacAddress = function () {
        let funcName = "getIPMacAddress";
        if (this._check()) {
            return ax.getIPMacAddress();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 获取本地计算机名。
     * @returns {string} 本地计算机名。
     */
    this.getLocalHostName = function () {
        let funcName = "getLocalHostName";
        if (this._check()) {
            return ax.getLocalHostName();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }

    /**
     * 打开about关于对话框。
     * @returns {*} 无
     */
    this.about = function () {
        let funcName = "about";
        if (this._check()) {
            return ax.about();
        } else {
            this.saveOpt(funcName, [])
            this._msg(funcName);
        }
    }
}
var _ = {};
var OFD = (function () {
    var tool = {
        _extend: function (defs, target) {
            var r = target;
            if (this._isNull(r)) {
                if (this._isArray(defs)) {
                    r = [];
                } else {
                    r = {};
                }
            }
            this._each(defs, function (n, v) {
                if (!(n in r)) {
                    r[n] = v;
                }
            });
            return r;
        },
        _isArray: function (v) {
            return Object.prototype.toString.call(v) === "[object Array]";
        },
        _isPlainObject: function (v) {
            if (!v || v.toString() !== "[object Object]" || v.nodeType
                || "setInterval" in v) {
                return false;
            }
            try {
                if (v.constructor && !v.hasOwnProperty("constructor")
                    && !v.constructor.prototype.hasOwnProperty("isPrototypeOf")) {
                    return false;
                }
            } catch (e) {
                return false;
            }
            var key;
            for (key in v) {
            }
            return key === undefined || v.hasOwnProperty(key);
        },
        _isNull: function (v) {
            return typeof v == "undefined" || (v !== 0 && !v);
        },
        _isValid: function (v) {
            return this._isNull(v) !== true;
        },
        _$: function (id) {
            return document.getElementById(id);
        },
        _new: function (tag) {
            return document.createElement(tag);
        },
        _each: function (o, fn) {
            if (this._isArray(o)) {
                for (var i = 0, ol = o.length, val = o[0]; i < ol
                && fn.call(val, i, val) !== false; val = o[++i]) {
                }
            } else {
                for (var i in o) {
                    if (fn.call(o[i], i, o[i]) === false) {
                        break;
                    }
                }
            }
            return o;
        },
        Page: {
            width: function () {
                var w = 0;
                if (window.innerWidth) {
                    w = window.innerWidth;
                } else if ((document.body) && (document.body.clientWidth)) {
                    w = document.body.clientWidth;
                }
                if (document.documentElement
                    && document.documentElement.clientHeight
                    && document.documentElement.clientWidth) {
                    w = document.documentElement.clientWidth;
                }
                return w;
            },
            height: function () {
                var h = 0;
                if (window.innerHeight) {
                    h = window.innerHeight;
                } else if ((document.body) && (document.body.clientHeight)) {
                    h = document.body.clientHeight;
                }
                if (document.documentElement
                    && document.documentElement.clientHeight
                    && document.documentElement.clientWidth) {
                    h = document.documentElement.clientHeight;
                }
                return h;
            },
            dEvent: function (e) {
                if (!e) {
                    return window.event;
                } else {
                    return e;
                }
            }
        }
    }

    function Reader(config) {
        var _p = new readyInit(this, config);
        _p.load()
        this.id = _p.id;
        _[this.id] = _p;
        this.func = new Funcs(_p)
    }

    Reader.prototype.valid = function () {
        return tool._isValid(_[this.id].ax)
    }
    Reader.prototype.version = function () {
        let embed = document.getElementById(this.id)
        try {
            return embed.getPluginVersion();
        } catch (e) {
        }
        return false;
    }

    let readyInit = function (reader, options) {
        this.id = "";
        this.opts = tool._extend({
            div: null,
            width: "100%",
            height: "100%",
            composite: {
                show: [],
                hide: []
            },
            loadMsg: "<span>正在加载控件，请稍候....</span>",
            checkInstalled: true
        }, options);
        // 控件对象
        this.ox = "";
        // 缓存用户操作.因为某些情况下,用户操作时,控件还没有初始化完毕
        this.optCache = new Map();
        this._randomString = function (l) {
            let x = "0123456789qwertyuioplkjhgfdsazxcvbnm";
            let tmp = "";
            for (let i = 0; i < l; i++) {
                tmp += x.charAt(Math.ceil(Math.random() * 100000000) % x.length);
            }
            return tmp;
        };

        this._writeOCX = function () {
            let w = this.opts.width;
            if (tool._isNull(w) || w === "auto") {
                w = "100%";
            }
            let h = this.opts.height;
            if (tool._isNull(h) || h === "auto") {
                h = "100%";
            }
            tool._$(this.opts.div).innerHTML = "<embed id='" + this.id
                + "' width='" + w
                + "' height='" + h
                + "' type='application/x-reader'"
                + ">";
        };

        this.load = function () {
            let rand = this._randomString(10);
            let div = tool._$(this.opts.div);
            if (tool._isNull(this.opts.div)) {
                div.innerHTML = "<span style='color:red;'>启动FireFox控件环境未准备好!</span>";
            }
            div.innerHTML = this.opts.loadMsg;
            this.id = "ofdReader_" + rand;
            this._writeOCX();

        }
        this.ready = function () {
            let o = tool._$(this.id);
            if (!o) {
                let div = tool._$(this.opts.div);
                div.innerHTML = "<span>ActiveX控件未正确初始化!</span>";
                return;
            }
            this.ox = o;
            this.optCache.forEach(function (value, funcName) {
                value.forEach(function (param) {
                    let length = 0;
                    for (let i = 0; i < param.length && typeof param[i] !== "undefined"; i++) {
                        length = i + 1;
                    }
                    console.log("缓存funcName:" + funcName + "(" + param + ")")

                    switch (length) {
                        case 0:
                            reader.func[funcName]()
                            break;
                        case 1:
                            reader.func[funcName](param[0])
                            break;
                        case 2:
                            reader.func[funcName](param[0], param[1])
                            break;
                        case 3:
                            reader.func[funcName](param[0], param[1], param[2])
                            break;
                        case 4:
                            reader.func[funcName](param[0], param[1], param[2], param[3])
                            break;
                        case 5:
                            reader.func[funcName](param[0], param[1], param[2], param[3], param[4])
                            break;
                        case 6:
                            reader.func[funcName](param[0], param[1], param[2], param[3], param[4], param[5])
                            break;
                    }
                })
            })
        };
    }
    let intervalCache = {};
    let isFinished = false;
    return {
        OCX: {
            init: function (div, width, height) {
                let config = {};
                let tmp = arguments.callee.toString().match(/\(.*?\)/)[0];
                let names = tmp.replace(/[()\s]/g, '').split(',');
                let array = Array.prototype.slice.call(arguments, 0);
                tool._each(array, function (i, e) {
                    if (tool._isValid(e)) {
                        if (tool._isPlainObject(e)) {
                            tool._extend(e, config);
                        } else {
                            config[names[i]] = e;
                        }
                    }
                });
                let r = new Reader(config)
                let T = this;
                intervalCache[r.id] = setInterval(function () {
                    if (r.version() !== false) {
                        isFinished = true
                        T.ready(r.id);
                    }
                }, 500);
                return r.func;
            },
            ready: function (id) {
                let _p = _[id];
                if (tool._isValid(_p)) {//能进入说明插件准备完毕，可以调用接口
                    clearInterval(intervalCache[id]);
                    _p.ready();
                }
                return _p;
            },
            initFinished: function () {
                return isFinished === true
            }
        }
    }
})()

//加载并初始化阅读器OCX控件--application/xReader-ofd
init = function (divID, width, height) {
    console.log("5.27")
    return OFD.OCX.init(divID, width, height);
}